﻿using System.Collections;
using System.Collections.Generic;
using System.Linq;
using UnityEngine;
using UnityEngine.UI;
using System.Threading.Tasks;

public class GroundSearch : Mod
{
    public static GroundSearch instance;
    AssetBundle bundle;
    GameObject UI;
    Image progress;
    Image progress_bg;
    Text text;
    AudioSource audio;
    bool isSearching;
    float searchDuration = 50;
    float successChance = 40;
    float minDistanceFromRecentSpot = 2;
    List<Vector3> recentSpots = new List<Vector3>();
    List<AudioClip> audioClips = new List<AudioClip>();
    List<string> items = new List<string>()
    {
        "mud_from_water",
        "Stone"
    };

    List<string> sentences = new List<string>()
    {
        "<color=#64c944>You found some mud.</color>",
        "<color=#64c944>You found a stone.</color>"
    };

    public IEnumerator Start()
    {
        instance = this;
        AssetBundleCreateRequest request = AssetBundle.LoadFromMemoryAsync(GetEmbeddedFileBytes("groundsearch.assets"));
        yield return request;
        bundle = request.assetBundle;
        UI = Instantiate(bundle.LoadAsset<GameObject>("GroundSearchUI"), transform);
        audio = UI.GetComponent<AudioSource>();
        audioClips.Add(bundle.LoadAsset<AudioClip>("sound1"));
        audioClips.Add(bundle.LoadAsset<AudioClip>("sound2"));
        audioClips.Add(bundle.LoadAsset<AudioClip>("sound3"));
        progress = UI.transform.Find("Bar").Find("Progress").GetComponent<Image>();
        progress_bg = UI.transform.Find("Bar").GetComponent<Image>();
        text = UI.GetComponentInChildren<Text>();
        UI.SetActive(false);
        StartCoroutine(FreeSpots());
    }

    public IEnumerator FreeSpots()
    {
        while (true)
        {
            yield return new WaitForSeconds(300);
            recentSpots.Clear();
        }
    }

    [ConsoleCommand("groundsearch", "Search for mud & rocks in the ground.")]
    public static void SearchCommand()
    {
        instance?.StartSearch();
    }

    public async void StartSearch()
    {
        Player ply = GHAPI.GetLocalPlayer();
        if (GHAPI.IsCurrentSceneGame() && ply != null && !ply.IsDead() && !ply.m_IsInAir)
        {
            if (!isSearching)
            {
                if (Camera.main.transform.rotation.eulerAngles.x < 35 || Camera.main.transform.rotation.eulerAngles.x > 65)
                {
                    isSearching = true;
                    UI.SetActive(true);
                    progress_bg.gameObject.SetActive(false);
                    progress.fillAmount = 0;
                    text.text = "<color=#a82a2a>You need to look the ground.</color>";

                    await Task.Delay(1000);
                    isSearching = false;
                    UI.SetActive(false);
                    progress.fillAmount = 0;
                    text.text = "You are searching...";
                    return;
                }
                bool alreadySearched = recentSpots.Where(x => Vector3.Distance(ply.transform.position, x) < minDistanceFromRecentSpot).Count() > 0;
                if (!alreadySearched)
                {
                    isSearching = true;
                    UI.SetActive(true);
                    progress_bg.gameObject.SetActive(true);
                    progress.fillAmount = 0;
                    text.text = "You are searching...";
                    int time = 0;
                    audio.PlayOneShot(audioClips[Random.Range(0, audioClips.Count)]);
                    while (time < searchDuration)
                    {
                        time++;
                        await Task.Delay(1);
                        progress.fillAmount = time / searchDuration;
                    }
                    progress_bg.gameObject.SetActive(false);
                    bool foundSomething = Random.Range(0, 100) <= successChance;
                    recentSpots.Add(ply.transform.position);
                    if (foundSomething)
                    {
                        int id = Random.Range(0, items.Count);
                        string potentialItem = items[id];
                        string sentencetext = sentences[id];
                        text.text = sentencetext;
                        Item item = ItemsManager.Get().CreateItem(potentialItem, false);
                        item.transform.position = ply.transform.position + (ply.transform.forward * 1f);
                    }
                    else
                    {
                        text.text = "<color=#a82a2a>There's nothing here.</color>";
                    }

                    await Task.Delay(2000);
                    isSearching = false;
                    UI.SetActive(false);
                    progress.fillAmount = 0;
                    text.text = "You are searching...";

                }
                else
                {
                    isSearching = true;
                    UI.SetActive(true);
                    progress_bg.gameObject.SetActive(false);
                    progress.fillAmount = 0;
                    text.text = "<color=#a82a2a>There's nothing more here.</color>";

                    await Task.Delay(2000);
                    isSearching = false;
                    UI.SetActive(false);
                    progress.fillAmount = 0;
                    text.text = "You are searching...";

                }
            }
            else
            {
                Debug.LogWarning("You are already searching the ground.");
            }
        }
        else
        {
            Debug.LogWarning("You can't search right now.");
        }
    }

    public void OnModUnload()
    {
        if (bundle != null)
            bundle.Unload(true);
    }
}